/*****************************************************************************************
/* File: IO.cpp
/* Desc: Class for handling inpung & drawing, it uses SDL for the rendering. Change the methods
/* of this class in order to use a different renderer
/*
/*
/* gametuto.com - Javier Lpez Lpez (javilop.com)
/*
/*****************************************************************************************
/*
/* Creative Commons - Attribution 3.0 Unported
/* You are free:
/*	to Share  to copy, distribute and transmit the work
/*	to Remix  to adapt the work
/*
/* Under the following conditions:
/* Attribution. You must attribute the work in the manner specified by the author or licensor
/* (but not in any way that suggests that they endorse you or your use of the work).
/*
/*****************************************************************************************/

// ------ Includes -----
#include <unistd.h>
#include "IO.h"

static SDL_Surface *mScreen;										// Screen
static Uint32 mColors [COLOR_MAX] = {0x000000ff,					// Colors
                               0xff0000ff, 0x00ff00ff, 0x0000ffff,
                               0x00ffffff, 0xff00ffff, 0xffff00ff,
                               0xffffffff};

void Terminate(void)
{
	SDL_Quit();
#ifdef GP2X
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif
}

/*
======================================
Init
======================================
*/
IO::IO()
:m_joy( NULL )
{
	InitGraph ();
}

IO::~IO()
{
    if( NULL != m_joy )
    {
        SDL_JoystickClose( m_joy );
        m_joy = NULL;
    }
}

/*
======================================
Clear the screen to black
======================================
*/
void IO::ClearScreen()
{
	boxColor (mScreen, 0, 0, mScreen->w - 1, mScreen->h - 1, mColors[BLACK]);
}


/*
======================================
Draw a rectangle of a given color

Parameters:
>> pX1, pY1: 		Upper left corner of the rectangle
>> pX2, pY2: 		Lower right corner of the rectangle
>> pC				Rectangle color
======================================
*/
void IO::DrawRectangle (int pX1, int pY1, int pX2, int pY2, enum color pC)
{
	boxColor (mScreen, pX1, pY1, pX2, pY2-1, mColors[pC]);
}


/*
======================================
Return the screen height
======================================
*/
int IO::GetScreenHeight()
{
	return mScreen->h;
}


/*
======================================
Update screen
======================================
*/
void IO::UpdateScreen()
{
	SDL_Flip(mScreen);
}


/*
======================================
Keyboard Input
======================================
*/
int IO::Pollkey()
{
	SDL_Event event;
	while ( SDL_PollEvent(&event) )
	{
		switch (event.type) {
		    case SDL_JOYAXISMOTION:
                if(SDL_JoystickGetAxis(m_joy, 0) < -32000)
                { // X Axis (Left)
                    return CAANOO_AXIS_LEFT;
                }
                else if(SDL_JoystickGetAxis(m_joy, 0) > +32000)
                { // X Axis (Right)
                    return CAANOO_AXIS_RIGHT;
                }
                break;
//                if(SDL_JoystickGetAxis(m_joy, 1) < -32000)
//                { // Y Axis (Up)
//                    return CAANOO_AXIS_UP;
//                }
//                else if(SDL_JoystickGetAxis(m_joy, 1) > +32000)
//                { // Y Axis (Down)
//                    return CAANOO_AXIS_DOWN;
//                }
		    case SDL_JOYBUTTONDOWN:
                return event.jbutton.button;
			case SDL_KEYDOWN:
				return event.key.keysym.sym;
			case SDL_QUIT:
				exit(3);
		}
	}
	return -1;
}

/*
======================================
Keyboard Input
======================================
*/
int IO::Getkey()
{
	SDL_Event event;
	while (true)
	{
	  SDL_WaitEvent(&event);
	  if (event.type == SDL_KEYDOWN || event.type == SDL_JOYBUTTONDOWN )
		  break;
      if (event.type == SDL_QUIT)
		  exit(3);
	};
	return event.jbutton.button;
	//return event.key.keysym.sym;
}

/*
======================================
Keyboard Input
======================================
*/
int IO::IsKeyDown (int pKey)
{
	Uint8* mKeytable;
	int mNumkeys;
	SDL_PumpEvents();
	mKeytable = SDL_GetKeyState(&mNumkeys);
	return mKeytable[pKey];
}

/*
======================================
Joypad Input
======================================
*/
int IO::IsButtonDown (int pBut)
{
	return SDL_JoystickGetButton( m_joy, pBut );
}

/*
======================================
SDL Graphical Initialization
======================================
*/
int IO::InitGraph()
{
	const SDL_VideoInfo *info;
	Uint8  video_bpp;
	Uint32 videoflags;

	// Initialize SDL
	if ( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_JOYSTICK ) < 0 ) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		return 1;
	}
	//atexit(SDL_Quit);
	atexit( Terminate );

	SDL_ShowCursor( SDL_DISABLE );

	// Alpha blending doesn't work well at 8-bit color
	info = SDL_GetVideoInfo();
	if ( info->vfmt->BitsPerPixel > 8 ) {
		video_bpp = info->vfmt->BitsPerPixel;
	} else {
		video_bpp = 16;
	}
	videoflags = SDL_SWSURFACE;// | SDL_DOUBLEBUF;

	// Set 320x240 video mode
	if ( (mScreen=SDL_SetVideoMode(320,240,video_bpp,videoflags)) == NULL ) {
		fprintf(stderr, "Couldn't set %ix%i video mode: %s\n",320,240,SDL_GetError());
		return 2;
	}

	if (SDL_NumJoysticks() > 0) {
		m_joy = SDL_JoystickOpen(0);
		if(!m_joy) {
			fprintf( stderr, "Couldn't open joystick 0: %s\n", SDL_GetError () );
		}
	}

    return 0;
}
